from hardcode_house_model.house.common import *
from hardcode_house_model.house.house_deal import *
from hardcode_house_model.house.house_price import *
from hardcode_house_model.house.spider_job import *


from enum import Enum

from hardcode_house_model.house.common import HouseLocation, HouseQuotation
from hardcode_house_model.statistics.common import ChangeDirection
from hardcode_house_model.util.mongo_mixin import DocumentMixin, MongoMixin
from mongoengine import Document
from mongoengine.fields import (DateTimeField, FloatField, DictField,
                                EmbeddedDocumentField, FloatField, IntField,
                                ListField, StringField, URLField)


class HouseEstateWeeklyReport(Document, MongoMixin, DocumentMixin):
    meta = {
        "strict": False,
        "indexes": [
            {"fields": ("city", "platform", "week_begin_date"), "unique": True},
        ]
    }

    city = StringField()
    platform = StringField(required=True)
    week_begin_date = DateTimeField()
    mean_price = FloatField()
    price_percentile25 = FloatField()
    price_percentile50 = FloatField()
    price_percentile75 = FloatField()
    price_percentile80 = FloatField()
    price_percentile90 = FloatField()
    price_percentile95 = FloatField()
    house_count = IntField()
    price_increase_count = IntField()
    price_decrease_count = IntField()
    newlisting_count = IntField()
    delisting_count = IntField()
    unchanged_count = IntField()
    updated_datetime = DateTimeField()

class HouseEstateWeeklyPrice(Document, MongoMixin, DocumentMixin):
    meta = {
        "strict": False,
        "indexes": [
            {"fields": ("report_id", "estate_name"), "unique": True},
            ("platform", "estate_name", "week_begin_date", "location.city", "location.district"),
            ("week_begin_date", "mean_price")
        ]
    }

    report_id = StringField(required=True)
    platform = StringField(required=True)
    estate_name = StringField()
    location = EmbeddedDocumentField(HouseLocation)
    week_begin_date = DateTimeField()
    mean_price = FloatField()
    price_percentile25 = FloatField()
    price_percentile50 = FloatField()
    price_percentile75 = FloatField()
    price_percentile80 = FloatField()
    price_percentile90 = FloatField()
    price_percentile95 = FloatField()
    house_count = IntField()
    price_increase_count = IntField()
    price_decrease_count = IntField()
    newlisting_count = IntField()
    delisting_count = IntField()
    unchanged_count = IntField()
    updated_datetime = DateTimeField()


class HouseEstateWeeklyPriceChange(Document, MongoMixin, DocumentMixin):
    meta = {
        "strict": False,
        "indexes": [
            {"fields": ("report_id", "platform_house_id"), "unique": True},
            ("platform_house_id", "week_begin_date", "location.city", "location.district"),
            ("location.city", "location.district", "week_begin_date"),
            ("direction", "delta.total_price"),
            ("direction", "delta.unit_price")
        ]
    }

    report_id = StringField(required=True)
    estate_name = StringField()
    location = EmbeddedDocumentField(HouseLocation)
    week_begin_date = DateTimeField()
    platform = StringField()
    platform_house_id = StringField()
    last_week_snapshot_date = DateTimeField()
    last_week_quotation = EmbeddedDocumentField(HouseQuotation)
    current_week_snapshot_date = DateTimeField()
    current_week_quotation = EmbeddedDocumentField(HouseQuotation)
    direction = IntField()
    delta = EmbeddedDocumentField(HouseQuotation)
    delta_percentage = FloatField()
    updated_datetime = DateTimeField()


class HouseEstateMonthlyPrice(Document, MongoMixin, DocumentMixin):
    meta = {
        "strict": False,
        "indexes": [
            {"fields": ("estate_name", "month_begin_date", "location.city", "location.district"), "unique": True},
            ("location.city", "location.district", "month_begin_date"),
            ("month_begin_date", "mean_price")
        ]
    }

    platform = StringField(required=True)
    estate_name = StringField()
    location = EmbeddedDocumentField(HouseLocation)
    month_begin_date = DateTimeField()
    mean_price = FloatField()
    price_percentile25 = FloatField()
    price_percentile50 = FloatField()
    price_percentile75 = FloatField()
    price_percentile80 = FloatField()
    price_percentile90 = FloatField()
    price_percentile95 = FloatField()
    house_count = IntField()
    updated_datetime = DateTimeField()


class HouseEstateMonthlyPriceChange(Document, MongoMixin, DocumentMixin):
    meta = {
        "strict": False,
        "indexes": [
            {"fields": ("platform_house_id", "month_begin_date", "location.city", "location.district"), "unique": True},
            ("location.city", "location.district", "month_begin_date"),
            ("direction", "delta.total_price"),
            ("direction", "delta.unit_price")
        ]
    }

    estate_name = StringField()
    location = EmbeddedDocumentField(HouseLocation)
    month_begin_date = DateTimeField()
    platform = StringField(required=True)
    platform_house_id = StringField(required=True)
    last_month_snapshot_date = DateTimeField(required=True)
    last_month_quotation = EmbeddedDocumentField(HouseQuotation)
    current_month_snapshot_date = DateTimeField(required=True)
    current_month_quotation = EmbeddedDocumentField(HouseQuotation)
    direction = IntField()
    delta = EmbeddedDocumentField(HouseQuotation)
    updated_datetime = DateTimeField()




from hardcode_house_model.house.common import (HouseLocation, HouseProperty,
                                               HouseQuotation,
                                               TransactionProperty)
from hardcode_house_model.util.mongo_mixin import DocumentMixin, MongoMixin
from mongoengine import Document
from mongoengine.document import EmbeddedDocument
from mongoengine.fields import (DateTimeField, FloatField, DictField,
                                EmbeddedDocumentField, FloatField, IntField,
                                ListField, StringField, URLField)


class HousePrice(Document, MongoMixin, DocumentMixin):
    meta = {
        "strict": False,
        "indexes": [
            {"fields": ("platform", "platform_house_id"), "unique": True},
            ("estate_name", "area"),
            ("built_datetime"),
            ("location.city", "location.district")
        ]
    }

    platform = StringField(required=True)
    platform_house_id = StringField(required=True)
    platform_title = StringField(required=True)
    platform_description = StringField(required=True)
    total_price = FloatField(required=True)
    unit_price = FloatField(required=True)
    quotation = EmbeddedDocumentField(HouseQuotation)
    area = FloatField()
    estate_name = StringField()
    built_datetime = DateTimeField()
    images = ListField(StringField())
    scrape_datetime = DateTimeField()
    url = URLField()
    house_property = EmbeddedDocumentField(HouseProperty)
    transaction_property = EmbeddedDocumentField(TransactionProperty)
    location = EmbeddedDocumentField(HouseLocation)

    created_datetime = DateTimeField()
    updated_datetime = DateTimeField(required=True)


class HousePriceHistory(Document, MongoMixin, DocumentMixin):
    meta = {
        "strict": False,
        "indexes": [
            {"fields": ("platform", "platform_house_id",
                        "snapshot_date"), "unique": True},
            ("calendar_year", "calendar_weeknumber", "calendar_weekday"),
            ("location.city", "location.district"),
            ("scrapy_job")
        ]
    }

    platform = StringField(required=True)
    platform_house_id = StringField(required=True)
    snapshot_date = DateTimeField(required=True)
    scrapy_job = StringField()
    calendar_year = IntField()
    calendar_weeknumber = IntField()
    calendar_weekday = IntField()
    total_price = FloatField()
    unit_price = FloatField()
    location = EmbeddedDocumentField(HouseLocation)
    quotation = EmbeddedDocumentField(HouseQuotation)
    area = FloatField()
    estate_name = StringField()
    status = IntField()
    updated_datetime = DateTimeField(required=True)

    def to_dict(self):
        return self.to_dict_default("%Y-%m-%d %H:%M:%S")



from enum import Enum

from hardcode_house_model.util.mongo_mixin import MongoMixin
from mongoengine import Document
from mongoengine.fields import (DateTimeField, FloatField, DictField,
                                EmbeddedDocumentField, FloatField, IntField,
                                ListField, StringField, URLField)


class ChangeDirection(Enum):
    Increase = 1
    Decrease = 2
    Newlisting = 3
    Delisting = 4




from mongoengine import Document
from mongoengine.fields import (BooleanField, DateTimeField, FloatField,
                                EmbeddedDocument, EmbeddedDocumentField,
                                FloatField, ListField, StringField, URLField)
from hardcode_house_model.util.mongo_mixin import MongoMixin

class HouseProperty(EmbeddedDocument, MongoMixin):
    house_type = StringField()      # 房屋户型
    floor = StringField()           # 所在楼层
    builtup_area = StringField()     # 建筑面积
    house_structure = StringField() # 户型结构
    usable_area = StringField()      # 套内面积
    building_type = StringField()   # 建筑类型
    orientation = StringField()     # 房屋朝向
    building_structure = StringField() # 建筑结构
    decoration = StringField()      # 装修情况
    lift_house_ratio = StringField()   # 梯户比例
    has_lift = BooleanField()       # 配备电梯


class TransactionProperty(EmbeddedDocument, MongoMixin):
    listing_time = DateTimeField()   # 挂牌时间
    transaction_type = StringField() # 交易属性
    last_transaction_time = DateTimeField() # 上次交易
    house_usage = StringField()      # 房屋用途
    hold_time = StringField()        # 房屋持有年限
    property_right = StringField()   # 产权所属
    mortgage = StringField()         # 抵押信息
    ownership_certificate = StringField() # 房本配备pass


class GeographicCoordinate(EmbeddedDocument, MongoMixin):
    latitude = FloatField()
    longitude = FloatField()
    altitude = FloatField()

    def __eq__(self, other):
        factor = 100000
        return int(self.latitude * factor) == int(other.latitude * factor) and int(self.longitude * factor) == int(other.longitude * factor)

    def __ne__(self, other):
        return not self.__eq__(other)

    def __hash__(self):
        factor = 100000
        return hash((int(self.latitude * factor), int(self.longitude * factor)))


class HouseLocation(EmbeddedDocument, MongoMixin):
    city = StringField()
    district = StringField()
    street = StringField()
    geographic_coordinate = EmbeddedDocumentField(GeographicCoordinate)


class Transaction(EmbeddedDocument, MongoMixin):
    unit_price = FloatField()
    total_price = FloatField()
    deal_date = DateTimeField()


class HouseQuotation(EmbeddedDocument, MongoMixin):
    total_price = FloatField(required=True)
    unit_price = FloatField(required=True)



from datetime import date, datetime
from enum import Enum

from bson import ObjectId


class MongoMixin(object):
    '''Contains various querying methods, useful for mongodb.'''
    class MongoUpdateMethod(Enum):
        SET = 'set'
        UNSET = 'unset'

    @classmethod
    def all_fields(cls):
        return cls._fields.keys()

    @classmethod
    def all_relationships(cls):
        return cls._relationships.keys()

    def resolved_relationships(self):
        return [
            a for a in self.all_relationships()
        ]

    def dict_include(self):
        return self.all_fields()

    def dict_exclude(self):
        return []

    # Default to_dict, can be overridden
    # Returns all fields, with values transformed:
    # -ObjectIds casted to strings
    # -Date/Datetime formatted as string, with format specified by
    # default_date_format property of class
    # -Values of lists and dicts recursively transformed
    def to_dict_default(self, date_format="%Y-%m-%d", ignore_unloaded=False):
        def transform_field(value):
            if isinstance(value, ObjectId):
                return str(value)
            elif isinstance(value, list):
                return map(transform_field, value)
            elif issubclass(value.__class__, MongoMixin):
                return value.to_dict() if hasattr(value, 'to_dict') else \
                    value.to_dict_default(ignore_unloaded=ignore_unloaded)
            elif isinstance(value, dict):
                return dict([(key, transform_field(val))
                             for key, val in value.iteritems()])
            elif isinstance(value, (datetime, date)):
                return value.strftime(date_format)
            return value

        _dict = {}
        exclude = self.dict_exclude()
        for fname in self.dict_include():
            if fname not in exclude:
                value = getattr(self, fname)
                _dict[fname] = transform_field(value)
        return _dict

    @classmethod
    def from_dict(cls, dict_data, created=False):
        return super()._from_son(dict_data, created=created)

    def to_dict(self):
        return self.to_dict_default("%Y-%m-%d %H:%M:%S")

class DocumentMixin(object):
    @classmethod
    def bulk_update(cls, requests, ordered=True, bypass_document_validation=False, session=None):
        collection = cls._get_collection()
        collection.bulk_write(
            requests, ordered, bypass_document_validation, session)



from hardcode_house_model.house.common import (HouseLocation, HouseProperty,
                                               Transaction,
                                               TransactionProperty)
from hardcode_house_model.util.mongo_mixin import DocumentMixin, MongoMixin
from mongoengine import Document
from mongoengine.fields import (DateTimeField, FloatField,
                                EmbeddedDocumentField, FloatField, IntField,
                                ListField, StringField, URLField)


class HouseDeal(Document, MongoMixin, DocumentMixin):
    platform = StringField(required=True)
    platform_house_id = StringField(required=True)
    platform_title = StringField(required=True)
    platform_description = StringField(required=True)
    total_deal_price = FloatField()                   # 成交总价
    total_listing_price = FloatField()                # 挂牌总价
    unit_deal_price = FloatField()                    # 成交单价
    listing_period = IntField()                       # 成交周期
    price_adjustment_times = IntField()               # 调价次数
    estate_name = StringField()
    images = ListField(StringField())
    scrape_datetime = DateTimeField()
    url = StringField()
    house_property = EmbeddedDocumentField(HouseProperty)
    transaction_property = EmbeddedDocumentField(TransactionProperty)
    transactions = ListField(EmbeddedDocumentField(Transaction, default=[]))
    location = EmbeddedDocumentField(HouseLocation)

    created_datetime = DateTimeField()
    updated_datetime = DateTimeField(required=True)

    def to_dict(self):
        return self.to_dict_default("%Y-%m-%d %H:%M:%S")



from pymongo.operations import InsertOne, UpdateOne, UpdateMany

class BulkOperationBuilder(object):
    def __init__(self):
        self.requests = []

    def insert_one(self):
        pass

    def update_one(self, filter, update, upsert):
        self.requests.append(UpdateOne(filter, update, upsert))

    def build(self):
        return self.requests



from enum import Enum

from hardcode_house_model.house.common import (HouseLocation, HouseProperty,
                                               HouseQuotation,
                                               TransactionProperty)
from hardcode_house_model.util.mongo_mixin import DocumentMixin, MongoMixin
from mongoengine import Document
from mongoengine.document import EmbeddedDocument
from mongoengine.fields import (DateTimeField, DictField,
                                EmbeddedDocumentField, FloatField, IntField,
                                ListField, StringField, URLField)


class SpiderJob(Document, MongoMixin):
    class Status(Enum):
        Running = 0
        Completed = 100

    class SpiderRunEnv(Enum):
        Zyte = "Zyte"
        Scrapyd = "Scrapyd"

    meta = {
        "strict": False,
        "indexes": [
            {"fields": ("spider", "job"), "unique": True},
            ("job", "spider_run_env"),
            ("status", "spider_run_env"),
            ("updated_datetime")
        ]
    }

    spider = StringField(required=True)
    job = StringField(required=True)
    spider_run_env = StringField(required=True)
    status = IntField(required=True)
    item_count = IntField(required=True)
    updated_datetime = DateTimeField(required=True)
